@@ -25,6 +25,10 @@
 #' maximum BG percent metric (default=12mm)
 #' @param bg_shrink amount to shrink the BG image away from the object to avoid
 #' residual object signal in the maximum BG percent metric (default=25mm)
+#' @param t1_canny threshold for weak edges for Canny edge detector (defaults to
+#' auto detect)
+#' @param t2_canny threshold for strong edges for Canny edge detector (defaults
+#' to auto detect)
 #' @return dataframe of QA metrics
 #' @examples
 #' fname <- system.file("extdata", "qa_data.nii.gz", package = "fmriqa")
@@ -51,7 +55,7 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
                    gen_res_csv = TRUE, gen_pdf = FALSE, gen_spec_csv = FALSE,
                    png_fname = NULL, res_fname = NULL, pdf_fname = NULL,
                    spec_fname = NULL, verbose = TRUE, bg_smooth = 12,
-                   bg_shrink = 25) {
+                   bg_shrink = 25, t1_canny = NULL, t2_canny = NULL) {
 
   if (is.null(data_file)) {
     filters <- matrix(c("NIfTI", ".nii.gz", "NIfTI", ".nii",
@@ -93,7 +97,6 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
     spec_file <- spec_fname
   }
 
-  #image_cols <- inferno(64)
   image_cols <- viridis(64)
 
   if (verbose) cat(paste("Reading data  : ", data_file, "\n\n", sep = ""))
@@ -178,9 +181,18 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
   # set na values to zero
   SFNR_full[is.na(SFNR_full)] <- 0
 
-  # threshold the image to reduce inhomogenity for cog calc
+  # threshold the image using edge detection to reduce inhomogenity for cog calc
   av_image_cimg <- imager::as.cimg(av_image)
-  obj_thr <- imager::threshold(av_image_cimg)
+
+  if (is.null(t1_canny)) {
+    obj_thr <- imager::px.flood(imager::as.cimg(
+               imager::cannyEdges(av_image_cimg)), x_dim / 2, y_dim / 2)
+  } else {
+    obj_thr <- imager::px.flood(imager::as.cimg(
+               imager::cannyEdges(av_image_cimg, t1 = t1_canny, t2 = t2_canny)),
+                                  x_dim / 2, y_dim / 2)
+  }
+
   cog_image <- obj_thr[,]
 
   # mean object intensity
@@ -188,10 +200,12 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
 
   # find max background intensity
   pix_dim <- min(c(x_pix_dim, y_pix_dim))
+
   # shrink BG to avoid object pixels
   obj_thr_bg <- imager::grow(obj_thr, round(bg_shrink/pix_dim))
   bg <- av_image_cimg
   bg[obj_thr_bg] <- 0
+
   # smooth BG
   bg_blur <- imager::boxblur(bg, round(bg_smooth/pix_dim))
   max_bg <- max(bg_blur)
@@ -214,6 +228,19 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
     y_pos <- round(y_pos)
   }
 
+  # measure image position as a function of time
+  #anal_vols <- dim(data_raw)[3]
+  #disp_array <- rep(NA, anal_vols)
+  #for (n in 1:anal_vols) {
+  #  image <- data_raw[,,n]
+  #  image <- imager::cannyEdges(imager::as.cimg(image))[,]
+  #  x_pos <- sum(array(1:x_dim, c(x_dim, y_dim)) * image) / sum(image)
+  #  y_pos <- sum(t(array(1:y_dim, c(y_dim, x_dim))) * image) / sum(image)
+  #  disp_array[n] = (x_pos ^ 2 + y_pos ^ 2) ^ 0.5
+  #}
+  #plot(disp_array)
+  #print(diff(range(disp_array)))
+
   # get ROI indices
   ROI_x <- get_pixel_range(x_pos, roi_width)
   ROI_y <- get_pixel_range(y_pos, roi_width)
@@ -376,7 +403,8 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
   raw_text <- paste(line1, line2, line3, line4, line5, line6, line7, line8,
                     line9, line10, line11, sep = "")
 
-  text <- textGrob(raw_text, x = 0.2, just = 0, gp = gpar(fontfamily = "mono", fontsize = 12))
+  text <- textGrob(raw_text, x = 0.2, just = 0, gp = gpar(fontfamily = "mono",
+                                                          fontsize = 12))
 
   # these are to appease R checks
   Measured <- NULL
@@ -390,14 +418,16 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
   value <- NULL
 
   # RDC plot
-  rdc_df <- data.frame(roi_width_vec = 1:roi_width, Theoretical = CV_ideal, Measured = CV)
+  rdc_df <- data.frame(roi_width_vec = 1:roi_width, Theoretical = CV_ideal,
+                       Measured = CV)
   rdc_df <- gather(rdc_df, group, CV, c(Measured, Theoretical))
 
-  rdc_plot <- ggplot(rdc_df, aes(x = roi_width_vec, y = CV, colour = group)) + geom_line() +
-    geom_point() + scale_x_log10(limits = c(1,100)) +
+  rdc_plot <- ggplot(rdc_df, aes(x = roi_width_vec, y = CV, colour = group)) +
+    geom_line() + geom_point() + scale_x_log10(limits = c(1,100)) +
     scale_y_log10(limits = c(0.01,10), breaks = c(0.01,0.1,1,10)) +
     labs(y = "100*CV", x = "ROI width (pixels)", title = "RDC plot") + marg +
-    theme(legend.position = c(0.8, 0.8)) + scale_color_manual(values = c("black","red"))
+    theme(legend.position = c(0.8, 0.8)) +
+    scale_color_manual(values = c("black","red"))
 
   tc_fit <- data.frame(t = vols, tc = mean_sig_intensity_t, fit = y_fit)
   tc_plot <- ggplot(tc_fit, aes(t)) + geom_line(aes(y = tc)) +
@@ -413,16 +443,16 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
     labs(y = "Intensity (a.u.)", x = "Time (volumes)",
          title = "BG intensity drift plot") + marg
 
-  spec_plot <- qplot(freq, spec, xlab = "Frequency (Hz)",
-                     ylab = "Intensity (a.u.)", geom = "line",
-                     main = "Fluctuation spectrum") + marg
+  spec_plot <- ggplot(mapping = aes(x = freq, y = spec)) + geom_line() +
+               labs(y = "Intensity (a.u.)", x = "Frequency (Hz)",
+               title = "Fluctuation spectrum") + marg
 
-  x_st = ROI_x[1]
-  x_end = ROI_x[length(ROI_x)]
-  y_st = ROI_y[1]
-  y_end = ROI_y[length(ROI_y)]
+  x_st  <- ROI_x[1]
+  x_end <- ROI_x[length(ROI_x)]
+  y_st  <- ROI_y[1]
+  y_end <- ROI_y[length(ROI_y)]
 
-  lcol <- "white"
+  lcol  <- "white"
 
   roi_a <- geom_segment(aes(x = x_st, xend = x_st, y = y_st, yend = y_end),
                         colour = lcol)
@@ -437,7 +467,6 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
                         colour = lcol)
 
 
-
   top_val <- quantile(SFNR_full,0.999)
   SFNR_full <- ifelse(SFNR_full > top_val, top_val, SFNR_full)
 
@@ -457,7 +486,8 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
     scale_fill_gradientn(colours = image_cols, na.value = "white") +
     coord_fixed(ratio = 1) + labs(x = "", y = "", fill = "Intensity",
                                   title = "Mean BG image") +
-    marg + scale_x_continuous(expand = c(0, 0)) + scale_y_continuous(expand = c(0, 0))
+    marg + scale_x_continuous(expand = c(0, 0)) +
+    scale_y_continuous(expand = c(0, 0))
 
   av_plot <- ggplot(melt(av_image), aes(Var1, Var2, fill = value)) +
     geom_raster(interpolate = TRUE) +
@@ -548,6 +578,38 @@ run_fmriqa <- function(data_file = NULL, roi_width = 21, slice_num = NULL,
   results_tab
 }
 
+#' Run fMRI quality assurance procedure on a set of NIfTI data files
+#'
+#' @param pattern glob expresion to match analysis files
+#' @param ... options to pass to run_fmriqa function
+#' @export
+run_fmriqa_glob <- function(pattern, ...) {
+  files <- Sys.glob(pattern)
+  res <- NULL
+  cat(paste(length(files), "file(s) found.\n"))
+  for (file in files) {
+    cat(file, "\n")
+    temp_res <- run_fmriqa(file, ...)
+    res <- rbind(res, temp_res)
+  }
+  res
+}
+
+#' Combine fmriqa csv result files into a dataframe
+#' @param pattern glob pattern to match the csv result files (eg "*.csv")
+#' @return a dataframe of results
+#' @export
+combine_res_glob <- function(pattern) {
+  files <- Sys.glob(pattern)
+  cat(paste(length(files), "file(s) found.\n"))
+  res <- NULL
+  for (file in files) {
+    temp_res <- utils::read.csv(file)
+    res <- rbind(res, temp_res)
+  }
+  res
+}
+
 #' @import RcppEigen
 detrend_fast <- function(y, X) {
   fastLmPure(y = y, X = X)$residual
